class TicketsController < ApplicationController

  def show
    @ticket = Ticket.find params[:id]
    @feature = Feature.find(@ticket.feature_id)
    @tickets = Ticket.find :all, :conditions=>"feature_id = #{params[:id]}",  :order => "state DESC"    
    @logs = Log.find_all_by_ticket_id @ticket.id, :order=>"created_at DESC"        
    @milestone = Milestone.find(@feature.milestone_id)
    respond_to do |format|
      format.html # show.html.erb
      format.xml  { render :xml => @ticket }
    end
  end
  
  def new   
    @feature = Feature.find(params[:feature_id])
    @ticket = Ticket.new
    @ticket.user_id = session[:user_id]
    @ticket.confirm_user_id = session[:user_id]
    @users = User.find_by_staff
    respond_to do |format|
      format.html # new.html.erb
      format.xml  { render :xml => @ticket }
    end
  end  
  
  def create
    @feature = Feature.find(params[:feature_id])
    @ticket = @feature.tickets.build(params[:ticket])
    @ticket.confirm_user_id = @ticket.user_id
    @ticket.created_by = session[:user_id]
    @ticket.state = Ticket::WAIT_STATE
    @ticket.body.strip!
    @ticket.body.gsub! /\s/, ' '
    @ticket.save!
    Log.ticket_created  @feature.id, session[:user_id], @ticket.id
    @users = User.find_by_staff    
    render :update do |page|
      page << "modal_wnd.destroy(false);"
      page.insert_html :bottom, :tickets, :partial=> @ticket
      page[ @ticket ].visual_effect :highlight
    end
  end

  def edit
    @ticket = Ticket.find(params[:id])
    @feature = Feature.find @ticket.feature_id, :select=>"milestone_id"
    @features = Feature.find_all_by_milestone_id @feature.milestone_id
    @milestones = Milestone.find :all    
    @users = User.find_by_staff
  end
  
  def update_feature_list
    @features = Feature.find_all_by_milestone_id params[:milestone_id]
    render :update do |page|
      page.replace_html 'feature_list', :partial=> "update_feature_list"
    end
  end
  
  def update
    @ticket = Ticket.find(params[:id])
    old_feature_id = @ticket.feature_id
    feature_id = old_feature_id 
    
    params[:ticket][:feature_id] = feature_id unless params[:ticket][:feature_id]
    
    if params[:ticket][:feature_id].to_i != old_feature_id
      feature_id = params[:ticket][:feature_id]
      Log.ticket_moved old_feature_id, feature_id, session[:user_id], @ticket.id
    end   
  
    unless params[:ticket][:end_date] == ""
      end_date = Date.parse( params[:ticket][:end_date] )
      if @ticket.end_date != end_date      
        Log.ticket_end_date_changed  feature_id, session[:user_id], @ticket.id, end_date 
      end
    end
   
    params[:ticket][:end_date] = Date.today.to_s(:db) if (@ticket.state != params[:ticket][:state]) and (params[:ticket][:state] == Ticket::DONE_STATE)
    
    if @ticket.state != params[:ticket][:state]
      Log.ticket_state_changed feature_id, session[:user_id], @ticket.id, params[:ticket][:state] 
    end    
    
    params[:ticket][:body].strip!
    @ticket.update_attributes!(params[:ticket])    
        
    render :update do |page|
      page.reload
      #page.redirect_to( :controller=>'features', :action => 'show', :id=> old_feature_id  )
    end
  end

  def update_state
    @ticket = Ticket.find(params[:id])
    @feature = Feature.find(@ticket.feature_id)
    
    params[:ticket] = {}
    params[:ticket][:state] =  params[:state] 
    params[:ticket][:start_date] = Date.today.to_s(:db) if params[:ticket][:state] == Ticket::WORK_STATE
    params[:ticket][:end_date] = Date.today.to_s(:db) if params[:ticket][:state] == Ticket::DONE_STATE
    
    @ticket.update_attributes!( params[:ticket] )    
    @users = User.find_by_staff
    
    Log.ticket_state_changed @feature.id, session[:user_id], @ticket.id, @ticket.state
    
    render :update do |page|
      if params[:reload]
        page.reload
      else
        page[@ticket].replace :partial=> @ticket
        page[@ticket].visual_effect :highlight
      end
    end
  end
  
  
  def user
    @ticket_partial_prefix = 'f'
    @tickets = Ticket.find_all_by_user_id params[:id], :order=>"created_at DESC"
  end
  
  
   def destroy
    @ticket = Ticket.find(params[:id])
    @ticket.destroy
    
    Log.delete_all "ticket_id = #{@ticket.id}"

    render :update do |page|
      page.reload      
    end
  end  
  
  
end

