class Milestone < ActiveRecord::Base
  include ApplicationHelper
  
  STATES = [
            ['계획중', 'f'],  #future
            ['실행중', 'w'], #working
            ['종료', 'd' ], #done
            ['폐기', 'a'] # aboart
  ]
  has_many :features
   
  def remaining_tickets
    tickets = []
    self.features.each do |f|
      tickets = tickets.concat( f.tickets.find(:all, :conditions => "state != 'd' and state != 'a'") )
    end
    tickets
  end
  
  def stat
      ticket_cnt = 0
      done_ticket_cnt = 0
      aborted_ticket_cnt = 0
      working_ticket_cnt = 0 
      pending_ticket_cnt = 0 
      waiting_ticket_cnt = 0
      delayed_ticket_cnt = 0
            
      self.features.each do |f|
        ticket_cnt += f.tickets.count
        done_ticket_cnt += f.tickets.count( :conditions=>"state='d'" )
        aborted_ticket_cnt += f.tickets.count( :conditions=>"state='a'" )
        working_ticket_cnt += f.tickets.count( :conditions=>"state='w'" )
        pending_ticket_cnt += f.tickets.count( :conditions=>"state='p'" )
        waiting_ticket_cnt += f.tickets.count( :conditions=>"state='t'" )
        delayed_ticket_cnt += f.tickets.count( :conditions=>"state='w' and end_date < '#{Date.today}'" )
      end
         
      def percent a,b
        return 0 if b == 0
        (a.to_f / b.to_f * 100.0).to_i
      end        
      
      valid_ticket_cnt = ticket_cnt - aborted_ticket_cnt
      progress = percent(done_ticket_cnt, valid_ticket_cnt)
      ideal_progress = 0
      if (self.start_date and self.end_date) and ( Date.today < self.end_date )        
        workday_all = workday_delta(self.start_date, self.end_date).to_f
        workday_cur = workday_delta(self.start_date, Date.today).to_f
        ideal_progress = percent(workday_cur, workday_all)
      end
      
      { 
        :progress=> progress,
        :ideal_progress => ideal_progress,        
        :ticket_count => ticket_cnt,        
        :done_ticket_count => done_ticket_cnt, 
        :aborted_ticket_count => aborted_ticket_cnt,
        :working_ticket_count => working_ticket_cnt, 
        :pending_ticket_count => pending_ticket_cnt, 
        :waiting_ticket_count => waiting_ticket_cnt,
        :delayed_ticket_count => delayed_ticket_cnt,
        :done_ticket_rate => percent( done_ticket_cnt, ticket_cnt),         
        :aborted_ticket_rate => percent( aborted_ticket_cnt, ticket_cnt),
        :working_ticket_rate => percent( working_ticket_cnt, ticket_cnt), 
        :pending_ticket_rate => percent( pending_ticket_cnt, ticket_cnt), 
        :waiting_ticket_rate => percent( waiting_ticket_cnt, ticket_cnt),
        :delayed_ticket_rate => percent( delayed_ticket_cnt, ticket_cnt)        
      }
  end    
  
end
