require 'prawn/core'
require "prawn/layout"
require 'prawn/table'
require 'pp'

include ApplicationHelper
include MyviewHelper

one_week_ago = (Date.today - 7).at_beginning_of_week
end_of_week = Date.today.at_end_of_week
done_tickets_at_last_week = Ticket.find :all, :conditions=>"(end_date BETWEEN '#{one_week_ago}' AND '#{one_week_ago.at_end_of_week}') and state ='d'"   
working_tickets = Ticket.find :all, :conditions=>"state ='w'", :order=>'feature_id'
delayed_tickets = Ticket.find :all, :conditions=>"state='w' and end_date < '#{Date.today}'", :order=>'feature_id'
working_staff_ids = User.find_by_working_staff_ids
waiting_tickets = Ticket.find_all_by_state 't'
less_waiting_staff_ids = waiting_tickets.inject(Hash.new(0)) {|memo, value| memo[value.user_id]+=1; memo}.sort {|a,b| (a[1]<=>b[1]) }.select { |x| x[1] < 3 and working_staff_ids.include?(x[0]) }
zero_waiting_staff_ids = (working_staff_ids - waiting_tickets.map { |t| t.user_id }).map { |u| [u,0] }
shortage_waiting_staff_ids = zero_waiting_staff_ids + less_waiting_staff_ids

not_working_staff_ids = working_staff_ids - working_tickets.map { |t| t.user_id } 
not_working_and_zero_waiting_staff_ids = not_working_staff_ids & zero_waiting_staff_ids.map { |u| u[0] }


metadata_info = {
  :Title => "Project LBO weekly report", :Author => "달다방", :Subject => "weekly report",
  :Keywords => "weekly report", :Creator => "달다방",
  :Producer => "Prawn", :CreationDate => Time.now, :Grok => "Confidential Property"
}

class Array
  def chunk(pieces)
    q, r = length.divmod(pieces)
    (0..pieces).map { |i| i * q + [r, i].min }.enum_cons(2).map { |a, b| slice(a...b) }
  end
end

def percent a,b
  (a.to_f / b.to_f * 100.0).to_i
end        


def get_pdf_path
  filename = "#{Date.today}-weekly_report.pdf"
  return File.join(ARGV[0],filename) if ARGV.size == 1
  filename
end

Prawn::Document.generate( get_pdf_path, :page_size => "A4", :info=>metadata_info) do
  
  def render_ticket_table tickets
    data = tickets.map do |t|
      [ user_name_from_id(t.user_id), t.body, date_fmt(t.start_date), date_fmt(t.end_date), date_duration(t.start_date,t.end_date) ]
    end         
    table data,
      :position           => 30,
      :font_size          => 6,
      #:headers            => ["담당자", "내용", "시작일", "종료일", "기간"],
      #:row_colors         => ["ffffff","ffff00"],
      :border_style => :underline_header,    
      :vertical_padding   => 2,
      :horizontal_padding => 3
  end    
    
  
  def render_tickets title, tickets
    move_down 20
    font "c:\\windows\\fonts\\h2hdrm.ttf", :size=>15
    text "#{title}(#{tickets.size}개)"
    move_down 10
    font "c:\\windows\\fonts\\h2gtrm.ttf", :size=>12   
    
    unless tickets.size
      text "없음"
      return
    end
    
    feature_ids = tickets.map { |t| t.feature_id }.uniq
    features = Feature.find :all, :conditions=>{ :id => feature_ids }, :order=>"feature_category_id"    
    ticket_group = tickets.group_by(&:feature_id).inject({}) { |m,v| m[v[0]] = v[1]; m }     
    feature_group = features.group_by(&:feature_category_id)    
    
    feature_group.each do |category_id,features|
      move_down(8)
      start_new_page if cursor() < 30
      text FeatureCategory.find( category_id,:select=>"name").name, :size=> 12
      move_down(8)      
      features.each do |f|
        start_new_page if cursor() < 20 
        span 350, :position => 15 do 
          text "F#{f.id} #{f.title}", :size => 9
        end
        start_new_page if cursor() < 20
        render_ticket_table ticket_group[f.id]
        move_down(5)
      end
    end       
  end
  
  def render_users title, user_ids, staffs_count
    move_down 20
    font "c:\\windows\\fonts\\h2hdrm.ttf", :size=>15
    text "#{title}(#{percent user_ids.size, staffs_count }%/#{user_ids.size}명)"
    move_down 10
    font "c:\\windows\\fonts\\h2gtrm.ttf", :size=>12
  
    unless user_ids.size
      text "없음"
      return
    end
    
    data = []
    if user_ids[0].is_a? Fixnum
      text user_ids.inject("") { |m,v| m << user_name_from_id(v) << " "; m }      
    else
      data = user_ids.group_by { |x| x[1] }.map { |a| ["#{a[0]}개"] + a[1].map { |c| user_name_from_id c[0] } }
=begin       
      table data,
      :position           => 10,
      :font_size          => 12,
      :border_style => :underline_header,    
      :vertical_padding   => 2,
      :horizontal_padding => 3
=end 
      span 350, :position=>15 do 
        data.each do |u|  
          text u.inject("") { |m,v| m << v << " "; m }, :size => 12
        end
      end
    end
     
  end
  
  footer [margin_box.left, margin_box.bottom] do
    move_down 20
    text "- #{page_count} -", :size => 10, :align =>:right
  end
 
  #Title
  font "c:\\windows\\fonts\\h2hdrm.ttf", :size=>28
  font_size 28
  text "#{date_fmt(Date.today)} 주보고", :align => :center
  stroke_horizontal_rule
  move_down 20


  milestones = Milestone.find_all_by_state 'w'
  
  milestones.each do |milestone|
    stat = milestone.stat
    font "c:\\windows\\fonts\\h2hdrm.ttf", :size=>15
    text "#{milestone.title}"
    move_down 10
    font "c:\\windows\\fonts\\h2gtrm.ttf", :size=>12    
    span(360, :postision=> :center) {
      text milestone_stat_text(milestone)
      text "총 티켓수: #{stat[:ticket_count]}개"
      text "총 완료된 티켓수: #{stat[:done_ticket_count]}개 (#{stat[:done_ticket_rate]}%)"
      text "총 지연된 티켓수: #{stat[:delayed_ticket_count]}개 (#{stat[:delayed_ticket_rate]}%)"
      text "총 작업중 티켓수: #{stat[:working_ticket_count]}개 (#{stat[:working_ticket_rate]}%)"
      text "총 대기중 티켓수: #{stat[:waiting_ticket_count]}개 (#{stat[:waiting_ticket_rate]}%)"
      text "총 보류된 티켓수: #{stat[:pending_ticket_count]}개 (#{stat[:pending_ticket_rate]}%)"    
      text "총 취소된 티켓수: #{stat[:aborted_ticket_count]}개 (#{stat[:aborted_ticket_rate]}%)"    
    }
    move_down 20
  end
  
  start_new_page
  render_users "작업 과 대기중 티켓이 없는 스탭", not_working_and_zero_waiting_staff_ids, working_staff_ids.size  
  render_users "대기중 티켓이 3개 미만인 스탭", shortage_waiting_staff_ids, working_staff_ids.size 
  

  render_tickets "지연된 티켓", delayed_tickets
  

  start_new_page
  render_tickets "지난주 완료된 티켓", done_tickets_at_last_week

  start_new_page
  render_tickets "작업중인 티켓", working_tickets
  
  
end
  
  